-- The complete UART module demonstrated on a DE0 board
-- 

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY UART_module_Demo_DE0 IS
	PORT (	
		OSC_CLK_in		: 	IN  STD_LOGIC;	
		CD_L			:	IN  STD_LOGIC;
		TX_IN			: 	IN 	STD_LOGIC_VECTOR (7 DOWNTO 0);		
		ST_L			: 	IN 	STD_LOGIC;
		RX_OUT			:	OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		CLK_1Hz_SQUARED	:	OUT STD_LOGIC -- The 1 Hz LED to show that the system works
	);
END UART_module_Demo_DE0 ;   
		

ARCHITECTURE structural OF UART_module_Demo_DE0  IS
-- The Four components
COMPONENT UART_module IS
	PORT (	
		OSC_CLK_in		: 	IN  STD_LOGIC;	
		CD				:	IN  STD_LOGIC;
		Enable_TX		:	IN  STD_LOGIC;
		TX_IN			: 	IN 	STD_LOGIC_VECTOR (7 DOWNTO 0);		
		ST				: 	IN 	STD_LOGIC;
		Serial_out		:	OUT  STD_LOGIC;
		ET				:  	OUT	STD_LOGIC;

		Enable_RX		:	IN  STD_LOGIC;
	   Serial_in   	:	IN  STD_LOGIC;
		RX_OUT			:	OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		Error			:	OUT STD_LOGIC;
		New_Data		:	OUT STD_LOGIC;
		CLK_1Hz_SQUARED	:	OUT STD_LOGIC -- The 1 Hz LED to show that the system works
	);
END COMPONENT;      


-- The signals to connect the modules:
	SIGNAL Serial_wire	: 	STD_LOGIC;   
	SIGNAL ET			: 	STD_LOGIC;   
	SIGNAL Error		: 	STD_LOGIC;  
	SIGNAL New_Data		: 	STD_LOGIC;   	
	SIGNAL ST, CD		:	STD_LOGIC;	-- To adapt the board's active-low push-buttons 
	
	
BEGIN
-- Instantiation of the components:
Chip1 : UART_module
	PORT MAP (
-- from component name 	=> to signal or port name
		OSC_CLK_in		=>	OSC_CLK_in,
		CD				=>  CD,
		Enable_TX		=> 	'1',
		TX_IN			=>	TX_IN,
		ST				=>  ST,
		Serial_out		=>	Serial_wire,
		ET				=>	ET,

		Enable_RX		=>	'1',
	   	Serial_in   	=>	Serial_wire,
		RX_OUT			=>	RX_OUT,
		Error			=>	Error,
		New_Data		=>	New_Data,
		
		CLK_1Hz_SQUARED	=>	CLK_1Hz_SQUARED
		
	);

-- Extra logic 

ST <= NOT (ST_L);           
CD <= NOT (CD_L);


END structural ;